@@ -235,35 +235,33 @@ static int lua_read_body(request_rec *r, const char **rbuf, apr_off_t *size,
 {
     int rc = OK;
 
-    *rbuf = NULL;
-    *size = 0;
-
     if ((rc = ap_setup_client_block(r, REQUEST_CHUNKED_ERROR))) {
         return (rc);
     }
     if (ap_should_client_block(r)) {
 
         /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
-        apr_off_t    len_read, rpos = 0;
+        char         argsbuffer[HUGE_STRING_LEN];
+        apr_off_t    rsize, len_read, rpos = 0;
         apr_off_t length = r->remaining;
         /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
         if (maxsize != 0 && length > maxsize) {
             return APR_EINCOMPLETE; /* Only room for incomplete data chunk :( */
         }
-        *rbuf = (const char *) apr_pcalloc(r->pool, (apr_size_t) (length) + 1);
-        while ((rpos < length)
-               && (len_read = ap_get_client_block(r, (char *) *rbuf + rpos,
-                                               length - rpos)) > 0) {
-            rpos += len_read;
+        *rbuf = (const char *) apr_pcalloc(r->pool, (apr_size_t) (length + 1));
+        *size = length;
+        while ((len_read = ap_get_client_block(r, argsbuffer, sizeof(argsbuffer))) > 0) {
+            if ((rpos + len_read) > length) {
+                rsize = length - rpos;
+            }
+            else {
+                rsize = len_read;
+            }
+
+            memcpy((char *) *rbuf + rpos, argsbuffer, (size_t) rsize);
+            rpos += rsize;
         }
-        if (len_read < 0) {
-            return APR_EINCOMPLETE;
-        }
-        *size = rpos;
-    }
-    else {
-        rc = DONE;
     }
 
     return (rc);
@@ -280,8 +278,6 @@ static apr_status_t lua_write_body(request_rec *r, apr_file_t *file, apr_off_t *
 {
     apr_status_t rc = OK;
 
-    *size = 0;
-
     if ((rc = ap_setup_client_block(r, REQUEST_CHUNKED_ERROR)))
         return rc;
     if (ap_should_client_block(r)) {
@@ -307,9 +303,6 @@ static apr_status_t lua_write_body(request_rec *r, apr_file_t *file, apr_off_t *
             rpos += rsize;
         }
     }
-    else {
-        rc = DONE;
-    }
 
     return rc;
 }
@@ -498,7 +491,7 @@ static int req_write(lua_State *L)
     return 1;
 }
 
-/* r:addoutputfilter(name|function) */
+/* r:add_output_filter(name) */
 static int req_add_output_filter(lua_State *L)
 {
     request_rec *r = ap_lua_check_request_rec(L, 1);
@@ -1283,6 +1276,10 @@ static int lua_ap_scoreboard_worker(lua_State *L)
         lua_pushstring(L, ws_record->client);
         lua_settable(L, -3);
 
+        lua_pushstring(L, "client64");
+        lua_pushstring(L, ws_record->client64);
+        lua_settable(L, -3);
+
         lua_pushstring(L, "conn_bytes");
         lua_pushnumber(L, (lua_Number) ws_record->conn_bytes);
         lua_settable(L, -3);
@@ -2189,20 +2186,23 @@ static int lua_websocket_greet(lua_State *L)
     return 0;
 }
 
-static apr_status_t lua_websocket_readbytes(conn_rec* c,
-                                            apr_bucket_brigade *brigade,
-                                            char* buffer, apr_off_t len)
+static apr_status_t lua_websocket_readbytes(conn_rec* c, char* buffer, 
+        apr_off_t len) 
 {
-    apr_size_t delivered;
+    apr_bucket_brigade *brigade = apr_brigade_create(c->pool, c->bucket_alloc);
     apr_status_t rv;
-
     rv = ap_get_brigade(c->input_filters, brigade, AP_MODE_READBYTES, 
             APR_BLOCK_READ, len);
     if (rv == APR_SUCCESS) {
-        delivered = len;
-        rv = apr_brigade_flatten(brigade, buffer, &delivered);
-        if ((rv == APR_SUCCESS) && (delivered < len)) {
-            rv = APR_INCOMPLETE;
+        if (!APR_BRIGADE_EMPTY(brigade)) {
+            apr_bucket* bucket = APR_BRIGADE_FIRST(brigade);
+            const char* data = NULL;
+            apr_size_t data_length = 0;
+            rv = apr_bucket_read(bucket, &data, &data_length, APR_BLOCK_READ);
+            if (rv == APR_SUCCESS) {
+                memcpy(buffer, data, len);
+            }
+            apr_bucket_delete(bucket);
         }
     }
     apr_brigade_cleanup(brigade);
@@ -2232,28 +2232,35 @@ static int lua_websocket_peek(lua_State *L)
 
 static int lua_websocket_read(lua_State *L) 
 {
+    apr_socket_t *sock;
     apr_status_t rv;
     int do_read = 1;
     int n = 0;
+    apr_size_t len = 1;
     apr_size_t plen = 0;
     unsigned short payload_short = 0;
     apr_uint64_t payload_long = 0;
     unsigned char *mask_bytes;
     char byte;
-    apr_bucket_brigade *brigade;
-    conn_rec* c;
-
+    int plaintext;
+    
+    
     request_rec *r = ap_lua_check_request_rec(L, 1);
-    c = r->connection;
+    plaintext = ap_lua_ssl_is_https(r->connection) ? 0 : 1;
 
+    
     mask_bytes = apr_pcalloc(r->pool, 4);
-
-    brigade = apr_brigade_create(r->pool, c->bucket_alloc);
+    sock = ap_get_conn_socket(r->connection);
 
     while (do_read) {
         do_read = 0;
         /* Get opcode and FIN bit */
-        rv = lua_websocket_readbytes(c, brigade, &byte, 1);
+        if (plaintext) {
+            rv = apr_socket_recv(sock, &byte, &len);
+        }
+        else {
+            rv = lua_websocket_readbytes(r->connection, &byte, 1);
+        }
         if (rv == APR_SUCCESS) {
             unsigned char ubyte, fin, opcode, mask, payload;
             ubyte = (unsigned char)byte;
@@ -2263,7 +2270,12 @@ static int lua_websocket_read(lua_State *L)
             opcode = ubyte & 0xf;
 
             /* Get the payload length and mask bit */
-            rv = lua_websocket_readbytes(c, brigade, &byte, 1);
+            if (plaintext) {
+                rv = apr_socket_recv(sock, &byte, &len);
+            }
+            else {
+                rv = lua_websocket_readbytes(r->connection, &byte, 1);
+            }
             if (rv == APR_SUCCESS) {
                 ubyte = (unsigned char)byte;
                 /* Mask is the first bit */
@@ -2274,25 +2286,40 @@ static int lua_websocket_read(lua_State *L)
 
                 /* Extended payload? */
                 if (payload == 126) {
-                    rv = lua_websocket_readbytes(c, brigade,
-                                                 (char*) &payload_short, 2);
+                    len = 2;
+                    if (plaintext) {
+                        /* XXX: apr_socket_recv does not receive len bits, only up to len bits! */
+                        rv = apr_socket_recv(sock, (char*) &payload_short, &len);
+                    }
+                    else {
+                        rv = lua_websocket_readbytes(r->connection, 
+                                (char*) &payload_short, 2);
+                    }
+                    payload_short = ntohs(payload_short);
 
-                    if (rv != APR_SUCCESS) {
+                    if (rv == APR_SUCCESS) {
+                        plen = payload_short;
+                    }
+                    else {
                         return 0;
                     }
-
-                    plen = ntohs(payload_short);
                 }
                 /* Super duper extended payload? */
                 if (payload == 127) {
-                    rv = lua_websocket_readbytes(c, brigade,
-                                                 (char*) &payload_long, 8);
-
-                    if (rv != APR_SUCCESS) {
+                    len = 8;
+                    if (plaintext) {
+                        rv = apr_socket_recv(sock, (char*) &payload_long, &len);
+                    }
+                    else {
+                        rv = lua_websocket_readbytes(r->connection, 
+                                (char*) &payload_long, 8);
+                    }
+                    if (rv == APR_SUCCESS) {
+                        plen = ap_ntoh64(&payload_long);
+                    }
+                    else {
                         return 0;
                     }
-
-                    plen = ap_ntoh64(&payload_long);
                 }
                 ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, APLOGNO(03210)
                               "Websocket: Reading %" APR_SIZE_T_FMT " (%s) bytes, masking is %s. %s", 
@@ -2301,27 +2328,46 @@ static int lua_websocket_read(lua_State *L)
                               mask ? "on" : "off", 
                               fin ? "This is a final frame" : "more to follow");
                 if (mask) {
-                    rv = lua_websocket_readbytes(c, brigade,
-                                                 (char*) mask_bytes, 4);
-
+                    len = 4;
+                    if (plaintext) {
+                        rv = apr_socket_recv(sock, (char*) mask_bytes, &len);
+                    }
+                    else {
+                        rv = lua_websocket_readbytes(r->connection, 
+                                (char*) mask_bytes, 4);
+                    }
                     if (rv != APR_SUCCESS) {
                         return 0;
                     }
                 }
                 if (plen < (HUGE_STRING_LEN*1024) && plen > 0) {
                     apr_size_t remaining = plen;
+                    apr_size_t received;
+                    apr_off_t at = 0;
                     char *buffer = apr_palloc(r->pool, plen+1);
                     buffer[plen] = 0;
 
-                    rv = lua_websocket_readbytes(c, brigade, buffer, remaining);
-
-                    if (rv != APR_SUCCESS) {
-                        return 0;
+                    if (plaintext) {
+                        while (remaining > 0) {
+                            received = remaining;
+                            rv = apr_socket_recv(sock, buffer+at, &received);
+                            if (received > 0 ) {
+                                remaining -= received;
+                                at += received;
+                            }
+                        }
+                        ap_log_rerror(APLOG_MARK, APLOG_TRACE1, 0, r, 
+                                "Websocket: Frame contained %" APR_OFF_T_FMT " bytes, pushed to Lua stack", 
+                                at);
+                    }
+                    else {
+                        rv = lua_websocket_readbytes(r->connection, buffer, 
+                                remaining);
+                        ap_log_rerror(APLOG_MARK, APLOG_TRACE1, 0, r, 
+                                "Websocket: SSL Frame contained %" APR_SIZE_T_FMT " bytes, "\
+                                "pushed to Lua stack", 
+                                remaining);
                     }
-
-                    ap_log_rerror(APLOG_MARK, APLOG_TRACE1, 0, r,
-                                  "Websocket: Frame contained %" APR_SIZE_T_FMT \
-                                  " bytes, pushed to Lua stack", remaining);
                     if (mask) {
                         for (n = 0; n < plen; n++) {
                             buffer[n] ^= mask_bytes[n%4];
@@ -2333,25 +2379,14 @@ static int lua_websocket_read(lua_State *L)
                     return 2;
                 }
 
+
                 /* Decide if we need to react to the opcode or not */
                 if (opcode == 0x09) { /* ping */
                     char frame[2];
-                    apr_bucket *b;
-
+                    plen = 2;
                     frame[0] = 0x8A;
                     frame[1] = 0;
-
-                    /* Pong! */
-                    b = apr_bucket_transient_create(frame, 2, c->bucket_alloc);
-                    APR_BRIGADE_INSERT_TAIL(brigade, b);
-
-                    rv = ap_pass_brigade(c->output_filters, brigade);
-                    apr_brigade_cleanup(brigade);
-
-                    if (rv != APR_SUCCESS) {
-                        return 0;
-                    }
-
+                    apr_socket_send(sock, frame, &plen); /* Pong! */
                     do_read = 1;
                 }
             }
